// honeywell_pressure.c
// (C) Bill Dally April, 2020
// Interface to Honeywell pressure sensor
// NPBMANN15DPGUNV

// Copyright 2020 Bill Dally
// 
// Licensed under the Ventilator Open Source License (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
// ./LICENSE
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// empirically output of 150psi sensor is 0.184 reading after zero correct

#define HP_150_CAL_64k 120586 // converts raw to 10s of PSI x 64k
#define HP_30_CAL_64k 24117 // for 30 PSI Sensor

// with 120586 16psi reads as 13.5
// so use - 142917

#define HP_30_CALX 142917

// set this line to select sensor
#define HP_CAL_64k (HP_30_CALX)

#include "main.h"
#include "adc.h"
#include "two_wire.h"
#include "nxp_pressure.h"
#include "honeywell_pressure.h"
#include <avr/interrupt.h>

uint16_t honeywell_press_zero = 0 ;

void honeywell_pressure_init(void) {
  honeywell_press_zero = read_at_word(EER_HP_ZERO) ;
  honeywell_press_2_zero = read_at_word(EER_HP2_ZERO) ;
}

void honeywell_set_zero(void) {
  honeywell_press_zero = get_adc(HONEYWELL_PRESS_ADC_CHAN) ;
  write_at_word(EER_HP_ZERO, honeywell_press_zero) ;
  honeywell_press_2_zero = get_adc(HONEYWELL_PRESS_2_ADC_CHAN) ;
  write_at_word(EER_HP2_ZERO, honeywell_press_2_zero) ;
}

// returns 10s of PSI
int16_t honeywell_get_pressure(void) {
  int16_t val ;
  val = get_adc(HONEYWELL_PRESS_ADC_CHAN) ;
  val = val - honeywell_press_zero ;
  if(val < 0) val = 0 ;
  int32_t lval = ((int32_t) val * HP_CAL_64k) >> 16 ;
  val = lval ;
  return(val) ;
}

// read second high-pressure sensor for oxygen
// returns 10s of PSI
int16_t honeywell_get_pressure2(void) {
  int16_t val ;
  val = get_adc(HONEYWELL_PRESS_2_ADC_CHAN) ;
  val = val - honeywell_press_2_zero ;
  if(val < 0) val = 0 ;
  int32_t lval = ((int32_t) val * HP_CAL_64k) >> 16 ;
  val = lval ;
  return(val) ;
}
