// mux.c
// Routines to drive analog mux to select flow meter
//
// (c) Bill Dally, April 2020
//
// Copyright 2020 Bill Dally
// 
// Licensed under the Ventilator Open Source License (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
// ./LICENSE
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#include <avr/io.h>
#include "main.h"
#include "honeywell_flow.h"


// set mux
void set_flow_mux(uint8_t sel) {
  if(BLEND == 1) { // only supported on blending units
    PORTC = (PORTC & ~(1<<FLOW_MUX1_PC)) | (((sel>>1) & 0x1)<<FLOW_MUX1_PC) ;
    PORTB = (PORTB & ~(1<<FLOW_MUX0_PB)) | ((sel & 0x1)<<FLOW_MUX0_PB) ;
  }
}

// get air flow
uint16_t get_air_flow() {
  uint16_t val ;
  set_flow_mux(0) ;
  val = honeywell_get_flow() ;
  return(val) ;
}


// get o2 flow
uint16_t get_o2_flow() {
  uint16_t val ;
  set_flow_mux(1) ;
  val = honeywell_get_flow() ;
  return(val) ;
}

// get aux flow
uint16_t get_aux_flow() {
  uint16_t val ;
  set_flow_mux(2) ;
  val = honeywell_get_flow() ;
  return(val) ;
}

// talk to each flow meter at start of time
void honeywell_flow_init() {
  uint8_t i ;
  for(i =0; i < NBR_FLOW_SENSORS ; i++) {
    set_flow_mux(i) ;
    honeywell_get_raw_flow() ;
    honeywell_get_raw_flow() ;
  }
}
